#!/bin/bash

PREFIX=/usr/local
MAKE_DEB='False'

debian_package_info()
{
cat <<EOF
### Commented entries have reasonable defaults.
### Uncomment to edit them.
# Source: <source package name; defaults to package name>
Section: misc
Priority: optional
# Homepage: <enter URL here; no default>
Standards-Version: 3.9.2

Package: anotherterminal
Version: 0.1
Maintainer: P.L. Lucas <selairi@gmail.com>
# Pre-Depends: <comma-separated list of packages>
Depends: python, gir1.2-vte-2.90
# Recommends: <comma-separated list of packages>
# Suggests: <comma-separated list of packages>
# Provides: <comma-separated list of packages>
# Replaces: <comma-separated list of packages>
# Architecture: all
Copyright: GPL3
# Changelog: <changelog file; defaults to a generic changelog>
# Readme: <README.Debian file; defaults to a generic one>
# Extra-Files: <comma-separated list of additional files for the doc directory>
#  <more pairs, if there's more than one file to include. Notice the starting space>
Description: AnotherTerminal is a terminal, like xterm, but menus can change to help you to use the Linux terminal.
 AnotherTerminal is a terminal, like xterm, for your Linux desktop. Menus of this terminal can change. Then menus can help you to use the Linux terminal. Moreover it can be used to provide a GUI to terminal applications. off your device before eject it.
 .
EOF

}

print_help()
{
cat <<EOF
$0 script installs AnotherTerminal in your system.
Arguments:
--prefix	Sets installation path (Default: $PREFIX)
--make-deb	Makes a Debian package
--help		Show this help.
EOF

}

build_launcher()
{
cat<<EOF
#!/bin/bash

ANOTHERTERMINAL_PATH=$1

export ANOTHERTERMINAL_PATH

exec "$3" -B -OO "\$ANOTHERTERMINAL_PATH"/"$2" "\$@"
EOF

}

#if [ $# -eq 0 ] ; then
#	print_help
#	exit 1
#fi
while [ $# -gt 0 ] ; do
	case $1 in
		--prefix) 
			PREFIX=$2
			shift 2 
			;;
		--make-deb) 
			MAKE_DEB=True
			shift 1
			;;
		--help) 
			print_help 
			exit 1
			;;
		*) shift 1 ;;
	esac
done


#Check python

if [[ $MAKE_DEB == 'False' ]]; then
	echo 'Checking python'
	PYTHON_EXEC=python3
	which python3
	if [[ $? -ne 0 ]]; then
		which python
		if [[ $? -ne 0 ]]; then
			echo 'Error: Python not installed'
			exit 1
		fi
		PYTHON_EXEC=python
	fi
fi

#Install

if [[ $MAKE_DEB == 'False' ]]; then
	echo 'Installing...'
else
	echo 'Making debian package...'
fi


BIN_PATH=$PREFIX/bin
SHARE_PATH=$PREFIX/share/anotherterminal
APP_PATH=$PREFIX/share/applications
ICONS_PATH=$PREFIX/share/icons

if [[ $MAKE_DEB == 'False' ]]; then
	echo 'Coping files...'
	cd src
	build_launcher "$SHARE_PATH" terminal3.py "$PYTHON_EXEC" > "$BIN_PATH"/anotherterminal
	build_launcher "$SHARE_PATH" builder.py "$PYTHON_EXEC" > "$BIN_PATH"/anotherterminal_builder
	
	mkdir -p "$SHARE_PATH"
	cp -R *.py *.xml menus icons "$SHARE_PATH"
	mkdir -p "$APP_PATH"
	cp anotherterminal.desktop "$APP_PATH"
	mkdir -p "$ICONS_PATH"
	cp icons/anotherterminal.svg "$ICONS_PATH"
	
	chmod o+r "$SHARE_PATH"
	chmod o+r "$APP_PATH"/anotherterminal.desktop
	chmod o+x "$BIN_PATH"/anotherterminal
	chmod o+x "$BIN_PATH"/anotherterminal_builder
	chmod o+r "$BIN_PATH"/anotherterminal
	chmod o+r "$BIN_PATH"/anotherterminal_builder
	chmod o+r "$ICONS_PATH"/anotherterminal.svg
	
	cd ..
else
	echo 'Building deb package...'
	cd src
	build_launcher "$SHARE_PATH" src/terminal3.py "$PYTHON_EXEC" > anotherterminal
	build_launcher "$SHARE_PATH" src/builder.py "$PYTHON_EXEC" > anotherterminal_builder
	
	chmod +x anotherterminal
	chmod +x anotherterminal_builder

	echo "Files:" anotherterminal.desktop "$APP_PATH" > ../scripts/files.txt
	find . -name '*.py' | awk "{print \"\", \$1, \"$SHARE_PATH\"}" >> ../scripts/files.txt
	ls *.xml | awk "{print \"\", \$1, \"$SHARE_PATH\"}" >> ../scripts/files.txt
	for lang in `ls menus` ; do
		find menus/$lang -name '*.xml' | awk "{print \"\", \$1, \"$SHARE_PATH\"}" >> ../scripts/files.txt
	done
	find icons -name '*.svg' | awk "{print \"\", \$1, \"$SHARE_PATH/\"}" >> ../scripts/files.txt
	echo "" anotherterminal "$BIN_PATH" >> ../scripts/files.txt
	echo "" anotherterminal_builder "$BIN_PATH" >> ../scripts/files.txt
	
	debian_package_info > ../scripts/anotherterminal.txt
	cat ../scripts/files.txt >> ../scripts/anotherterminal.txt
	
	equivs-build ../scripts/anotherterminal.txt
	mv *.deb ..
	cd ..
fi

#Uninstaller

if [[ $MAKE_DEB == 'False' ]]; then
echo 'Write uninstaller...'

cat<<EOF > uninstall.sh
#!/bin/bash

rm -Rf "$SHARE_PATH"
rm "$APP_PATH/anotherterminal.desktop"
rm "$BIN_PATH/anotherterminal"
rm "$BIN_PATH/anotherterminal_builder"
rm "$ICONS_PATH/anotherterminal.svg"
EOF

fi

echo 'The end'
