#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2012  P.L. Lucas
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import os.path
import glob
import sys
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, GObject, Gio, Pango, GdkPixbuf, Gtk, Gdk, Vte


config_path='/usr/share/anotherterminal'
if 'ANOTHERTERMINAL_PATH' in os.environ:
	config_path=os.environ['ANOTHERTERMINAL_PATH']

sys.path.append(config_path)


class OpenMenuDialog:
	def __init__(self, terminal):
		self.anotherterminal_builder='{python} "{config_path}/builder.py"'.format(python=sys.executable, config_path=config_path)
		self.terminal=terminal
		self.builder = Gtk.Builder()
		if self.builder.add_from_file(config_path+'/open_menu.xml')==0:
			print ("Failed to load UI XML file: open_menu.xml")
			sys.exit(1)
		#handlers = {
		#	"random_background_checkbutton_toggled_callback": self.random_background_checkbutton_toggled_callback
		#}
		#self.builder.connect_signals(handlers)
		self.dialog = self.builder.get_object("dialog")
		self.init_treeview()
		self.fill_treeview()
		ok=self.dialog.run()
		if ok==-5:	#Open menu
			self.open_menu()
		elif ok==-6:	#Edit menu
			menu_file=self.get_row()
			if menu_file!=None:
				os.system(self.anotherterminal_builder+' "{0}" &'.format(menu_file))
		elif ok==-7:	#New menu
			os.system(self.anotherterminal_builder+' &')
		self.dialog.destroy()
	
	def init_treeview(self):
		scrolledwindow=self.builder.get_object('scrolledwindow')
		self.treestore = Gtk.ListStore(str, object)
		self.treeview=Gtk.TreeView(self.treestore)
		self.treeview.show()
		scrolledwindow.add(self.treeview)
		
		renderer = Gtk.CellRendererText()
		column = Gtk.TreeViewColumn("Menu", renderer, text=0)
		column.set_expand(True)
		self.treeview.append_column(column)
		
		select = self.treeview.get_selection()
		self.treeview.connect("row-activated", self.row_activated_callback)
	
	def fill_treeview(self):
		list=[]
		# Get language:
		lang=os.environ['LANG'][0:2]
		menu_path='{path}/.config/AnotherTerminal/menus/{lang}/'.format(path=os.environ['HOME'], lang=lang)
		for f in self.list_menus(menu_path):
			list.append(f)
		menu_path='{path}/menus/{lang}/'.format(path=config_path, lang=lang)
		for f in self.list_menus(menu_path):
			list.append(f)
		menu_path='{path}/menus/{lang}/'.format(path=config_path, lang='en')
		for f in self.list_menus(menu_path):
			list.append(f)
		for f in list:
			self.treestore.append(f)
		
	
	def list_menus(self, path):
		if os.path.exists(path):
			list=[]
			for f in glob.glob(path+'/*.xml'):
				list.append([f,f])
			return list
		return []
	
	def row_activated_callback(self, path, column, data):
		self.open_menu()
		return True
	
	def get_row(self):
		selection = self.treeview.get_selection()
		model, treeiter = selection.get_selected()
		if treeiter != None and model[treeiter][1]!=None:
			#Open menuitem dialog
			return model[treeiter][1]
		return None
	
	def open_menu(self):
		menu_file=self.get_row()
		self.dialog.destroy()
		if menu_file!=None:
			self.terminal.open_xml(menu_file, False)
				
	