#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2012  P.L. Lucas
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import random
import os
import os.path
import sys
import subprocess
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, GObject, Gio, Pango, GdkPixbuf, Gtk, Gdk, Vte


config_path='/usr/share/anotherterminal'
if 'ANOTHERTERMINAL_PATH' in os.environ:
	config_path=os.environ['ANOTHERTERMINAL_PATH']

sys.path.append(config_path)


class PropertiesSettingsDialog:
	def __init__(self, terminal):
		self.builder = Gtk.Builder()
		
		if self.builder.add_from_file(config_path+'/propertiessettingsdialog.xml')==0:
			print ("Failed to load UI XML file: propertiesdialog.xml")
			sys.exit(1)
		handlers = {
			"random_background_checkbutton_toggled_callback": self.random_background_checkbutton_toggled_callback
		}
		self.builder.connect_signals(handlers)
		dialog = self.builder.get_object("dialog")
		random_background_checkbutton = self.builder.get_object("random_background_checkbutton")
		random_background_checkbutton.set_active(terminal.random_background_ok)
		random_white_background_radiobutton = self.builder.get_object("random_white_background_radiobutton")
		random_black_background_radiobutton = self.builder.get_object("random_black_background_radiobutton")
		if terminal.random_background_scheme=='black':
			random_black_background_radiobutton.set_active(True)
		else:
			random_white_background_radiobutton.set_active(True)
		foreground_colorbutton = self.builder.get_object("foreground_colorbutton")
		foreground_colorbutton.set_color(terminal.foreground_color)
		background_colorbutton = self.builder.get_object("background_colorbutton")
		background_colorbutton.set_color(terminal.background_color)
		autoupdate_menu_checkbutton = self.builder.get_object("autoupdate_menu_checkbutton")
		if terminal.updade_time>0:
			autoupdate_menu_checkbutton.set_active(True)
		else:
			autoupdate_menu_checkbutton.set_active(False)
		fontbutton = self.builder.get_object("fontbutton")
		if terminal.font!=None:
			fontbutton.set_font_name(terminal.font)
		self.random_background_checkbutton_toggled_callback(None)
		ok=dialog.run()
		if ok==-5:
			terminal.random_background_ok=random_background_checkbutton.get_active()
			if random_black_background_radiobutton.get_active():
				terminal.random_background_scheme='black'
			else:
				terminal.random_background_scheme='white'
			terminal.foreground_color=foreground_colorbutton.get_color()
			terminal.background_color=background_colorbutton.get_color()
			if autoupdate_menu_checkbutton.get_active():
				terminal.updade_time=5
			else:
				terminal.updade_time=-1
			terminal.font=fontbutton.get_font_name()
			terminal.update_properties()
			terminal.save_properties()
		dialog.destroy()
	
	def random_background_checkbutton_toggled_callback(self, data):
		foreground_colorbutton = self.builder.get_object("foreground_colorbutton")
		background_colorbutton = self.builder.get_object("background_colorbutton")
		foreground_label = self.builder.get_object("foreground_label")
		background_label = self.builder.get_object("background_label")
		foreground_colorbutton = self.builder.get_object("foreground_colorbutton")
		random_white_background_radiobutton = self.builder.get_object("random_white_background_radiobutton")
		random_black_background_radiobutton = self.builder.get_object("random_black_background_radiobutton")
		random_background_checkbutton = self.builder.get_object("random_background_checkbutton")
		ok=not random_background_checkbutton.get_active()
		foreground_colorbutton.set_sensitive(ok)
		background_colorbutton.set_sensitive(ok)
		foreground_label.set_sensitive(ok)
		background_label.set_sensitive(ok)
		random_white_background_radiobutton.set_sensitive(not ok)
		random_black_background_radiobutton.set_sensitive(not ok)
		return True
