#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2012  P.L. Lucas
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import random
import os
import os.path
import sys
import subprocess
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GLib, GObject, Gio, Pango, GdkPixbuf, Gtk, Gdk, Vte


config_path='/usr/share/anotherterminal'
if 'ANOTHERTERMINAL_PATH' in os.environ:
	config_path=os.environ['ANOTHERTERMINAL_PATH']

sys.path.append(config_path)

from projectxmlhandler import *
from pstree import *
from propertiessettingsdialog import *
from open_menu import *

class Terminal:
	"""This class build a new window with menu, toolbar and terminal.
	"""
	
	def __init__(self, opened_terminals_windows_list=[]):
		# Default config
		self.child_pid=-1
		self.timer_id=-1
		self.last_child=-1
		self.updade_time=5
		self.foreground_color=None
		self.background_color=None
		self.random_background_ok=True
		self.random_background_scheme='black'
		self.font=None
		self.opened_terminals_windows_list=opened_terminals_windows_list
		
		self.opened_terminals_windows_list.append(self)
		
		#Load user config
		self._load_config()
		
		#Build vte terminal
		self.terminal = Vte.Terminal()
		self.terminal.set_size_request(50,50)
		self.terminal.set_scrollback_lines(1000000)
		self.terminal.set_size(10,10)
		self.update_properties()
		self.terminal.connect("child-exited", self.delete_event)
		self.terminal.connect("focus-in-event", self.focus_in_callback)
		self.terminal.connect("focus-out-event", self.focus_out_callback)
		self.terminal.connect("resize-window", self.resize_window_callback)
		self.terminal.connect("button-press-event", self.button_press_event_callback)
		
		#Build new window
		self.window = Gtk.Window()
		self.window.set_icon(GdkPixbuf.Pixbuf.new_from_file(config_path+'/icons/anotherterminal.svg'))
		self.window.set_title("Terminal")
		self.window.set_size_request(400, 300)
		self.window.connect("delete_event", self.delete_event)
		
		hints=Gdk.Geometry()
		hints.base_width=self.terminal.get_char_width()
		hints.base_height=self.terminal.get_char_height()
		hints.min_width=self.terminal.get_char_width()
		hints.min_height=self.terminal.get_char_height()
		hints.width_inc=self.terminal.get_char_width()
		hints.height_inc=self.terminal.get_char_height()
		self.window.set_geometry_hints(None, hints, Gdk.WindowHints.BASE_SIZE|Gdk.WindowHints.MIN_SIZE|Gdk.WindowHints.RESIZE_INC)
		
		#Set up initial menus
		self.ui_manager=Gtk.UIManager()
		self._insert_menu(None)
		
		self.vbox= Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
		self.window.add(self.vbox)
		self.vbox.show()
		
		self._insert_menu_and_toolbar_widgets()
		
		# Insert terminal widget in window
		scrollbar=Gtk.VScrollbar()
		scrollbar.set_adjustment(Gtk.Scrollable.get_vadjustment(self.terminal))
		hbox=Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
		hbox.pack_start(self.terminal, True, True, 0)
		hbox.pack_end(scrollbar, False, False, 0)
		self.vbox.pack_end(hbox, True, True, 0)
		self.terminal.show()
		
		self.window.show_all()
	
	def save_properties(self):
		fin=None
		path=os.environ['HOME']+'/.config/AnotherTerminal/config.rc'
		if not os.path.exists(path):
			os.system('mkdir -p "{0}"'.format(os.environ['HOME']+'/.config/AnotherTerminal'))
		try:
			fin=open(path,'w')
		except:
			return False
		fin.write('random_background_ok={0}\n'.format(self.random_background_ok))
		fin.write('random_background_scheme={0}\n'.format(self.random_background_scheme))
		fin.write('foreground_color={0}\n'.format(self.foreground_color.to_string()))
		fin.write('background_color={0}\n'.format(self.background_color.to_string()))
		if self.font!=None:
			fin.write('font={0}\n'.format(self.font))
		if self.updade_time>0:
			fin.write('autoupdate_menu=True\n')
		else:
			fin.write('autoupdate_menu=False\n')
		fin.close()
	
	def update_properties(self):
		if self.random_background_ok:
			if self.random_background_scheme=='black':
				self.foreground_color=Gdk.Color.parse('snow')[1]
				background_colors=['#000000', '#000025', '#002500', '#250000', '#252500', '#250025', '#002525', '#252525']
			else: # if self.random_background_scheme=='white':
				self.foreground_color=Gdk.Color.parse('black')[1]
				background_colors=['#ffffff', '#ffffee', '#ffeeff', '#eeffff', '#eeeeff', '#eeffee', '#ffeeee', '#eeeeee']
			self.background_color=Gdk.Color.parse(background_colors[random.randint(0, len(background_colors)-1)])[1]
		self.terminal.set_colors(self.foreground_color, self.background_color, [])
		if self.font!=None:
			self.terminal.set_font(Pango.FontDescription.from_string(self.font))
	
	def _load_config(self):
		fin=None
		try:
			fin=open( os.environ['HOME']+'/.config/AnotherTerminal/config.rc','r')
		except:
			return False
		for line in fin:
			try:
				pos=line.index('=')
				name=line[:pos]
				value=line[pos+1:].strip()
				if name=='random_background_ok':
					if value=='True':
						self.random_background_ok=True
					else:
						self.random_background_ok=False
				elif name=='random_background_scheme':
					self.random_background_scheme=value
				elif name=='foreground_color':
					self.foreground_color=Gdk.Color.parse(value)[1]
				elif name=='background_color':
					self.background_color=Gdk.Color.parse(value)[1]
				elif name=='font':
					self.font=value
				elif name=='autoupdate_menu':
					if value=='True':
						self.updade_time=5
					else:
						self.updade_time=-1
			except:
				continue
		fin.close()
		return True
	
	def _insert_menu_and_toolbar_widgets(self):
		menubar = self.ui_manager.get_widget("/MenuBar")
		self.vbox.pack_start(menubar, False, False, 0)
		menubar.show()
		toolbar = self.ui_manager.get_widget("/ToolBar")
		if toolbar!=None:
			self.vbox.pack_start(toolbar, False, False, 0)
			toolbar.show()
	
	def exec_command(self, arguments):
		"""Run command, child, in terminal.
		
		arguments: child's argument list. Example: ["ssh",'-l',"usuario","host"]
		"""
		# List of process and their parents: ps -o user,pid,ppid,command -ax
		ok, self.child_pid = self.terminal.fork_command_full(Vte.PtyFlags.DEFAULT, None, arguments, [], GLib.SpawnFlags.SEARCH_PATH, None, None)
	
	def delete_event(self, widget, event=None, data=None):
		"""Close AnotherTerminal app.
		"""
		self.window.destroy()
		self.window=None
		self.opened_terminals_windows_list.remove(self)
		if len(self.opened_terminals_windows_list)==0:
			Gtk.main_quit()
		return False
	
	def resize_window_callback(self, width, height, data):
		self.terminal.set_size(int(width/self.terminal.get_char_width()), int(height/self.terminal.get_char_height()) )
	
	def update_terminal_size(self):
		width=self.terminal.get_allocated_width()
		height=self.terminal.get_allocated_height()
		self.resize_window_callback(width, height, None)
		#self.terminal.set_size(self.terminal.get_column_count(), self.terminal.get_row_count() )
	
	def _insert_menu(self, menuitems, toolbaritems=None):
		"""Insert a new menu and toolbar.
		
		This method takes items from XML handler and insert them in 
		menubar and toolbar.
		
		menuitems: are items to insert in menubar.
		toolbaritems: are items to insert in toolbar.
		"""
		ui_str = """
<ui>
  <menubar name='MenuBar'>
    <menu action='FileMenu'>
      <menuitem action='FileNew' />
      <menuitem action='FileOpen' />
      <menuitem action='FileConvert' />
      <separator />
      <menuitem action='FileQuit' />
    </menu>
    """
		
		# Get the Gtk.UIManager
		manager = self.ui_manager

		# Create a new action group
		self._action_group = Gtk.ActionGroup("ExamplePyPluginActions")
		self._add_action_to_group(self._action_group, Gtk.Action("FileNew", None, None, Gtk.STOCK_NEW), self.new_callback, "File new" )
		self._add_action_to_group(self._action_group, Gtk.Action("FileOpen", None, None, Gtk.STOCK_OPEN), self.open_callback, "File open" )
		self._add_action_to_group(self._action_group, Gtk.Action("FileConvert", None, None, Gtk.STOCK_CONVERT), self.convert_callback, "File open" )
		self._add_action_to_group(self._action_group, Gtk.Action("FileQuit", None, None, Gtk.STOCK_QUIT), self.delete_event, "File new" )
		self._action_group.add_actions(
			[
			("FileMenu", Gtk.STOCK_FILE, None, None, None, None)
			]
		)
		
		# Merge Edit menu
		ui_str+=self._add_edit_menu()
		
		# Merge the UI
		if menuitems!=None:
			ui_str+=self._build_ui_menu(menuitems, False)
		ui_str+='</menubar>\n'
		
		# Merge the toolbar
		if toolbaritems!=None:
			ui_str+="<toolbar name='ToolBar'>\n"
			ui_str+=self._build_ui_menu(toolbaritems, True)
			ui_str+='</toolbar>\n'
		
		# Merge popup menu
		ui_str+=self._add_popup_menu()
		
		ui_str+='</ui>'
		
		#print(ui_str)
		
		# Insert the action group
		manager.insert_action_group(self._action_group, -1)
		self._ui_id = manager.add_ui_from_string(ui_str)
		
		self.popup= manager.get_widget("/PopupMenu")
	
	def _edit_menu_content(self):
		"""XML text of edit menu.
		"""
		ui_str = """
      <menuitem action='EditCopy' />
      <menuitem action='EditPaste' />
      <separator />
      <menuitem action='EditUpdateMenus' />
"""
		self._add_action_to_group(self._action_group, Gtk.Action("EditCopy", None, None, Gtk.STOCK_COPY), self.copy_callback, None )
		self._add_action_to_group(self._action_group, Gtk.Action("EditPaste", None, None, Gtk.STOCK_PASTE), self.paste_callback, None )
		self._add_action_to_group(self._action_group, Gtk.Action("EditUpdateMenus", None, None, Gtk.STOCK_REFRESH), self.update_menus_callback, None )
		return ui_str
	
	def _add_popup_menu(self):
		"""Insert edit menu in menubar.
		"""
		ui_str = "<popup action='PopupMenu'>{0}</popup>".format(self._edit_menu_content())
		
		return ui_str

	def _add_edit_menu(self):
		"""Insert edit menu in menubar.
		"""
		ui_str = """
		<menu action='EditMenu'>
			{0}
			<separator />
			<menuitem action='EditProperties' />
		</menu>
		""".format(self._edit_menu_content())
		self._action_group.add_actions(
			[
			("EditMenu", Gtk.STOCK_EDIT, None, None, None, None)
			]
		)
		
		#self._add_action_to_group(self._action_group, Gtk.Action("EditMenuEdit", None, None, Gtk.STOCK_EDIT), self.menu_edit_callback, None )
		self._add_action_to_group(self._action_group, Gtk.Action("EditProperties", None, None, Gtk.STOCK_PROPERTIES), self.properties_callback, None )
		
		return ui_str

	def _add_action_to_group(self, action_group, action, callback_func, data):
		"""This helper add GtkAction to GtkActionGroup.
		"""
		name=action.get_name()
		if None==action_group.get_action(name):
			action_group.add_action(action)
			action.connect("activate", callback_func, data)

	def _remove_menu(self):
		# Get the Gtk.UIManager
		manager = self.ui_manager

		# Remove the ui
		manager.remove_ui(self._ui_id)

		# Remove the action group
		manager.remove_action_group(self._action_group)

		# Make sure the manager updates
		manager.ensure_update()
	
	def open_callback(self, action, data):
		mode=Gtk.FileChooserAction.OPEN
		dialog = Gtk.FileChooserDialog('Open', None, mode, (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
		response = dialog.run()
		if response==Gtk.ResponseType.OK:
			path=dialog.get_filename()
			self.open_xml(path, False)
		dialog.destroy()
		return False
	
	def convert_callback(self, action, data):
		openMenuDialog=OpenMenuDialog(self)
		return False
	
	def new_callback(self, action, data):
		terminal = Terminal()
		terminal.exec_command(['bash'])
		return False
	
	#def menu_edit_callback(self, action, data):
	#	os.system('anotherterminal_builder &')
	#	return True
		
	def properties_callback(self, action, data):
		properties=PropertiesSettingsDialog(self)
		return True
	
	
	def button_press_event_callback(self, widget, event):
		# Check if right mouse button was preseed
		if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 3:
			self.popup.popup(None, None, None, None, event.button, event.time)
			return True # event has been handled
		return False
	
	def focus_out_callback(self, event, data):
		self.timer_id=-1
		return False
	
	def focus_in_callback(self, event, data):
		if self.updade_time>0:
			self.timer_id=GLib.timeout_add_seconds(self.updade_time, self.update_menus_callback)
		return False
	
	def feed_callback(self, action, data):
		"""This callback send data to terminal.
		
		This method feeds terminal with data. As a virtual user who
		type in keyboard.
		"""
		self.terminal.feed_child(data, len(data) )
	
	def file_dialog_callback(self, action, data):
		"""This callback send data to terminal and open a file dialog.
		
		This method feeds terminal with data. As a virtual user who
		type in keyboard.
		"""
		#print('File dialog')
		mode=None
		if data.file_dialog_mode=='open':
			#mode=Gtk.FileChooserAction.OPEN
			mode=Gtk.FileChooserAction.SELECT_FOLDER
		elif data.file_dialog_mode=='save':
			mode=Gtk.FileChooserAction.SAVE
		dialog = Gtk.FileChooserDialog(data.name, None, mode, (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
		response = dialog.run()
		if response==Gtk.ResponseType.OK and data.command!=None:
			output=data.command.replace(data.file_dialog_replace_str, dialog.get_filename())
			if data.exec_ok:
				self.exec_callback(None, output)
			else:
				self.feed_callback(None, output)
		dialog.destroy()
	
	def exec_callback(self, action, data):
		"""This callback execute data as python code and send output
		to terminal.
		
		Data is executed as python code. There a special variable,
		called output. output variable is used to feed terminal.
		"""
		ns={}
		exec(data, ns)
		if 'output' in ns:
			output=str(ns['output'])
			self.feed_callback(None, output)
	
	def copy_callback(self, action, data):
		"""Copy selection to clipborad.
		"""
		self.terminal.copy_clipboard()
	
	def paste_callback(self, action, data):
		"""Feed terminal will data from clipboard.
		"""
		self.terminal.paste_clipboard()
	
	def update_menus_callback(self, action=None, data=None):
		"""Update menus of window.
		
		It checks process running in terminal and updates menus.
		"""
		child_data=pstree_get_last_child(self.child_pid)
		command=None
		if child_data!=None:
			# There is a child. It have to find menus
			# Get command
			if self.last_child!=child_data.pid:
				self.last_child=child_data.pid
				command=child_data.cmdline.split('\0')[0]
			else:
				command=None
		else:
			if self.last_child!=self.child_pid:
				self.last_child=self.child_pid
				command=pstree_read_file('/proc/'+str(self.child_pid)+'/cmdline').split('\0')[0]
			else:
				command=None
		if command!=None:
			# Get language:
			lang=os.environ['LANG'][0:2]
			menu_file='{path}/.config/AnotherTerminal/menus/{lang}/{command}.xml'.format(path=os.environ['HOME'], lang=lang, command=command)
			#print(menu_file)
			if not os.path.exists(menu_file):
				menu_file='{path}/menus/{lang}/{command}.xml'.format(path=config_path, lang=lang, command=command)
			if not os.path.exists(menu_file):
				lang='en'
				menu_file='{path}/.config/AnotherTerminal/menus/{lang}/{command}.xml'.format(path=os.environ['HOME'], lang=lang, command=command)
				#print(menu_file)
				if not os.path.exists(menu_file):
					menu_file='{path}/menus/{lang}/{command}.xml'.format(path=config_path, lang=lang, command=command)
			if os.path.exists(menu_file):
				self.open_xml(menu_file, False)
		
		#self.update_terminal_size()
		
		if self.timer_id>0:
			return True
		return False
	
	def open_xml(self, file_name, run_ok=True):
		"""Open XML file with menus.
		
		Structure of XML file can be read from documentation.
		"""
		# Process XML file
		parser = make_parser() 
		handler=ProjectXMLHandler()
		parser.setContentHandler(handler)
		parser.parse(open(file_name))
		
		# Build new menu
		self._remove_menu()
		self._insert_menu(handler.menubar, handler.toolbar)
		self.ui_manager.ensure_update()
		
		# Insert menu and toolbar in window
		self._insert_menu_and_toolbar_widgets()
		
		# Start child program in terminal
		if run_ok:
			self.exec_command(handler.command.split())
		
		# Set window title
		if handler.title!=None:
			self.window.set_title(handler.title)
		
	def _build_ui_menu(self, menuitem, toolbar_ok):
		"""This helper builds menus, toolbar and GtkActions for GtKUIManager.
		"""
		#print("_build_ui_menu")
		ui=''
		if isinstance(menuitem, MenuItem):
			if toolbar_ok:
				ui+="<toolitem action='{0}' />\n".format(menuitem.id)
			else:
				ui+="<menuitem action='{0}' />\n".format(menuitem.id)
			tooltip=menuitem.tooltip
			if tooltip==None:
				tooltip=menuitem.name
			action=Gtk.Action(menuitem.id, menuitem.name, tooltip, menuitem.stockicon)
			if menuitem.file_dialog_mode!=None:
				self._add_action_to_group(self._action_group, action, self.file_dialog_callback, menuitem )
			elif menuitem.exec_ok:
				self._add_action_to_group(self._action_group, action, self.exec_callback, menuitem.command )
			else:
				self._add_action_to_group(self._action_group, action, self.feed_callback, menuitem.command )
			
		elif isinstance(menuitem, Menu):
			ui+="<menu action='{0}'>\n".format(menuitem.id)
			self._action_group.add_actions(
				[
				(menuitem.id, None, menuitem.name, None, None, None)
				]
			)
			for item in menuitem.menuitems:
				ui+=self._build_ui_menu(item, toolbar_ok)
			ui+="</menu>\n"
		elif isinstance(menuitem, Separator):
			ui+="<separator />\n"
		elif isinstance(menuitem, Menubar) or isinstance(menuitem, Toolbar):
			for item in menuitem.menuitems:
				ui+=self._build_ui_menu(item, toolbar_ok)
		return ui

if __name__ == "__main__":
	filename=None
	command=None
	update_menu=True
	n=1
	while n<len(sys.argv):
		argv=sys.argv[n]
		n+=1
		if argv=='-e':
			command=sys.argv[n:]
			break
		elif argv=='--no-update-menu':
			update_menu=False
		elif argv=='--help':
			print("""
{0} [--help] [filename] [-e command args ...]
AnotherTerminal is a terminal emulator.
Options:
--help	Show this help
--no-update-menu No update menus.
-e      This option specifies the program (and its command line arguments) to be  run  in the window.
""".format(sys.argv[0]))
			exit(0)
		else:
			filename=argv
	terminal = Terminal()
	if not update_menu:
		terminal.updade_time=-1
	if filename!=None:
		terminal.updade_time=-1
		terminal.open_xml(filename)
	elif command!=None:
		terminal.exec_command(command)
	else:
		terminal.exec_command(['bash'])

	Gtk.main()

